using Microsoft.VisualStudio.TestTools.UnitTesting;
using CSIP_42.Model;
using System;
using System.Collections.Generic;
using System.ComponentModel.Design;
using System.Diagnostics;
using System.Linq;
using CSIP_42.Model.Entity;
using CSIP_42.Model.Entity.Games;
using CSIP_42.Model.Enumerators;
using Satsuma;

namespace CSIP_42.Model.Construction.Test
{
    [TestClass]
    public class ConstructionTest
    {
        public ConstructionTest()
        {
            allEntites = null;
            allStaff = null;
            notPlaceables = null;
            placeableInfrastructures = null;
        }

        private List<TypeNameEnum> allEntites;
        private List<TypeNameEnum> allStaff;
        private List<TypeNameEnum> notPlaceables;
        private List<TypeNameEnum> placeableInfrastructures;

        private void InitLists()
        {
            //Listing staff and placeables
            allEntites = new List<TypeNameEnum>(Enum.GetValues<TypeNameEnum>());
            allStaff = new List<TypeNameEnum>
            {
                TypeNameEnum.Cleaner, TypeNameEnum.Mechanic
            };
            notPlaceables = new List<TypeNameEnum>
            {
                TypeNameEnum.Guest, //Not placeable
                TypeNameEnum.MainGate, //Not placeable
                TypeNameEnum.FenceBasic,
                TypeNameEnum.FenceCorner
            };
            foreach (var staff in allStaff)
            {
                notPlaceables.Add(staff);
            }

            //Only placeable infrastructures
            placeableInfrastructures = new List<TypeNameEnum>(allEntites.Except(notPlaceables));
        }


        [TestMethod] //Tester class
        public void TestInitList()
        {
            Assert.IsNull(allEntites);
            Assert.IsNull(allStaff);
            Assert.IsNull(notPlaceables);
            Assert.IsNull(placeableInfrastructures);
            InitLists();
            Assert.IsNotNull(allEntites);
            Assert.IsNotNull(allStaff);
            Assert.IsNotNull(notPlaceables);
            Assert.IsNotNull(placeableInfrastructures);
        }

        [TestMethod] //Trivial
        public void Constructor()
        {
            Model model = new Model(true, "EntityConfigForConstruction.json");
            GameObject info = model.GetGameInfo();
            Assert.AreEqual(info.mainGate.GatePosition, model.GetMainGateGatePos());
            Assert.AreEqual(info.mainGate.Identity.SizeM, model.GetMainGateSize().Item1);
            Assert.AreEqual(info.mainGate.Identity.SizeN, model.GetMainGateSize().Item2);

            EntityIdentifier mainGateID = model.GetEntityIdentifier(TypeNameEnum.MainGate);
            Assert.AreEqual(mainGateID.Type, info.mainGate.Identity.Type);
        }

        [TestMethod] //Trivial
        [ExpectedException(typeof(KeyNotFoundException), "The given key 'Road' was not present in the dictionary")]
        public void GappyJSON_Constructor()
        {
            Model model = new Model(true, "EntityConfigForConstruction_NotAllEntities.json");
        }

        [TestMethod] //Trivial
        public void NewGame()
        {
            Model.N = 45;
            Model.M = 29;
            Model model = new Model(true, "EntityConfigForConstruction.json");
            model.NewGame();
            Assert.AreEqual(model.GetSize(), new Tuple<int, int>(45, 29));
            Assert.IsTrue(model.GetListOfPlaceable().Count > 0);
            Assert.IsTrue(model.GetMoney() > 0);

            var list = model.GetListOfPlaceable();
            Assert.IsTrue(list.Count > 0);

            InitLists();
            foreach (var enumerator in placeableInfrastructures)
            {
                Assert.IsNotNull(list.Find((e) => e.Type == enumerator));
            }
        }

        [TestMethod] //Non-Trivial
        public void AreaEmpty()
        {
            Model model = new Model(true, "EntityConfigForConstruction.json");
            model.NewGame();
            Assert.AreEqual(model.GetSize(), new Tuple<int, int>(45, 29));
            Assert.IsTrue(model.GetListOfPlaceable().Count > 0);
            Assert.IsTrue(model.GetMoney() > 0);

            var list = model.GetListOfPlaceable();
            Assert.IsTrue(list.Count > 0);

            int index = 0;
            //Placing first 1x1 Entity
            model.SelectForCurrentElement(TypeNameEnum.Bin);
            Position pos1 = new Position(5, 5);
            Assert.IsTrue(model.AreaEmpty(pos1));
            Assert.IsTrue(model.PlaceEntity(pos1, index++));
            Assert.IsFalse(model.AreaEmpty(pos1));
            model.SelectForCurrentElement(TypeNameEnum.FlowerPlant);
            Assert.IsFalse(model.AreaEmpty(pos1));

            //Placing second 1x1 Entity
            Position pos2 = new Position(4, 10);
            Assert.IsFalse(model.AreaEmpty(pos1));
            Assert.IsTrue(model.AreaEmpty(pos2));
            Assert.IsTrue(model.PlaceEntity(pos2, index++));
            Assert.IsFalse(model.AreaEmpty(pos2));
            model.SelectForCurrentElement(TypeNameEnum.TreePlant);
            Assert.IsFalse(model.AreaEmpty(pos2));

            //Placing a bigger Entity
            model.SelectForCurrentElement(TypeNameEnum.CarouselGame);
            Position pos3 = new Position(4, 4); //With this position it would overlay with the first placed Entity
            Position pos3_ok = new Position(6, 7);
            Assert.IsFalse(model.AreaEmpty(pos1));
            Assert.IsFalse(model.AreaEmpty(pos3));
            Assert.IsTrue(model.AreaEmpty(pos3_ok));
            Assert.IsTrue(model.PlaceEntity(pos3_ok, index++));
            Assert.IsFalse(model.AreaEmpty(pos3_ok));
            model.SelectForCurrentElement(TypeNameEnum.FlowerPlant);
            Assert.IsFalse(model.AreaEmpty(pos3_ok));


            //There are no road on table. Check
            var info = model.GetGameInfo();
            Assert.IsNotNull(info);
            Position acceptableRoadPos = new Position(model.GetMainGateGatePos().X, model.GetMainGateGatePos().Y - 1);
            Assert.AreEqual(info.Infrastructures[CategoryEnum.Road].Count, 1);
            Assert.IsTrue(info.Table[acceptableRoadPos.X, acceptableRoadPos.Y] is Road);

            for (int i = 0; i < info.SizeN; i++)
            {
                for (int j = 0; j < info.SizeM; j++)
                {

                    if (i == acceptableRoadPos.X && j == acceptableRoadPos.Y)
                        continue;

                    model.SelectForCurrentElement(TypeNameEnum.Cleaner);
                    Assert.IsFalse(model.AreaEmpty(new Position(i, j)));
                    model.SelectForCurrentElement(TypeNameEnum.Mechanic);
                    Assert.IsFalse(model.AreaEmpty(new Position(i, j)));
                }
            }

            //Building a road
            model.SelectForCurrentElement(TypeNameEnum.Road);
            for (int i = 0; i < 10; i++)
            {
                Position p = new Position(10 + i, 5);
                Assert.IsTrue(model.AreaEmpty(p));
                Assert.IsTrue(model.PlaceEntity(p, index++));
            }

            InitLists();
            for (int i = 0; i < 10; i++)
            {
                Position p = new Position(10 + i, 5);
                foreach (var enumerator in allStaff)
                {
                    model.SelectForCurrentElement(enumerator);
                    Assert.IsTrue(model.AreaEmpty(p));
                }
            }

        }

        [TestMethod] //Non-Trivial
        public void PlaceInfrastructure()
        {
            Model.N = 45;
            Model.M = 29;
            Model.StartMoney = 10000000;
            //Construction
            Model model = new Model(true, "EntityConfigForConstruction.json");
            model.NewGame();
            Assert.AreEqual(model.GetSize(), new Tuple<int, int>(45, 29));
            Assert.IsTrue(model.GetListOfPlaceable().Count > 0);
            Assert.IsTrue(model.GetMoney() > 0);
            var list = model.GetListOfPlaceable();
            Assert.IsTrue(list.Count > 0);

            //Placing 1x1 Entity
            int index = 0;
            model.SelectForCurrentElement(TypeNameEnum.Bin);
            Position pos1 = new Position(5, 5);
            var info = model.GetEntitiyInfoForPosition(pos1);
            Assert.IsNull(info);

            long preMoney = model.GetMoney();
            Assert.IsTrue(model.PlaceEntity(pos1, index++));
            long afterMoney = model.GetMoney();

            Assert.IsFalse(model.AreaEmpty(pos1));
            Assert.AreEqual(preMoney, afterMoney + model.GetEntityIdentifier(TypeNameEnum.Bin).PriceToPlace);

            InitLists();
            foreach (var enumerator in placeableInfrastructures)
            {
                model.SelectForCurrentElement(enumerator);
                Assert.IsFalse(model.PlaceEntity(pos1, index - 1));
            }

            model.SelectForCurrentElement(TypeNameEnum.CarouselGame);
            Position pos3 = new Position(4, 4); //Shoudl overlay
            Position pos3_ok = new Position(6, 7);
            Assert.IsFalse(model.PlaceEntity(pos3, index - 1));

            preMoney = model.GetMoney();
            Assert.IsTrue(model.PlaceEntity(pos3_ok, index++));
            afterMoney = model.GetMoney();

            Assert.AreEqual(preMoney, afterMoney + model.GetEntityIdentifier(TypeNameEnum.CarouselGame).PriceToPlace);
            Assert.IsFalse(model.AreaEmpty(pos3_ok));

            //Rotate ellen?rz�se
            model.SelectForCurrentElement(Enumerators.TypeNameEnum.GiantWheelGame);
            var preRot = model.CurrentID.ConstructionInformation.RotationTimes;
            model.RotateCurrentElement();
            var afterRot = model.CurrentID.ConstructionInformation.RotationTimes;
            Assert.AreNotEqual(preRot, afterRot);
            model.RotateCurrentElement();
            model.RotateCurrentElement();
            model.RotateCurrentElement();
            afterRot = model.CurrentID.ConstructionInformation.RotationTimes;
            Assert.AreEqual(preRot, afterRot);

            model.PlaceEntity(new Position(20, 20), index++);
            Assert.IsNotNull(model.GetGameInfo().Table[20, 20]);
            Assert.AreEqual(model.GetGameInfo().Table[20, 20].Identity.ConstructionInformation.RotationTimes, afterRot);
            model.SetEntryPriceForCurrentElement(1000);
            Assert.AreEqual(model.GetEntryPriceForCurrentElement(), 1000);
            model.SetMinCapacityForCurrentElement(50);
            model.SelectForModify(new Position(20, 20));
            EntityIdentifier ID = model.CurrentID;
            Assert.AreEqual(model.GetMinCapacityForCurrentElement(),
                (Convert.ToInt32(Math.Round((ID.ConstructionInformation.MinimumCapacity / (float) ID.Capacity) *
                                            100))));

            int buildTime = model.GetEntityIdentifier(TypeNameEnum.GiantWheelGame).TimeToBuild;
            for (int i = 0; i < buildTime; i++)
            {
                model.TestTick();
            }

            Assert.IsNotNull(model.GetGameInfo().Infrastructures[CategoryEnum.Game]
                .Find(e => e.Type == TypeNameEnum.GiantWheelGame));

        }

        [TestMethod] //Non-Trivial
        public void PlaceStaff()
        {
            Model.N = 45;
            Model.M = 29;
            Model.StartMoney = 100000000;
            //Construction
            Model model = new Model(true, "EntityConfigForConstruction.json");
            model.NewGame();
            Assert.AreEqual(model.GetSize(), new Tuple<int, int>(45, 29));
            Assert.IsTrue(model.GetListOfPlaceable().Count > 0);
            Assert.IsTrue(model.GetMoney() > 0);
            var list = model.GetListOfPlaceable();
            Assert.IsTrue(list.Count > 0);

            //Building a road
            int index = 0;
            model.SelectForCurrentElement(TypeNameEnum.Road);
            for (int i = 0; i < 10; i++)
            {
                Position pos1 = new Position(8 + i, 5);
                var info = model.GetEntitiyInfoForPosition(pos1);
                Assert.IsNull(info);

                long preMoney = model.GetMoney();
                Assert.IsTrue(model.PlaceEntity(pos1, index++));
                long afterMoney = model.GetMoney();

                Assert.IsFalse(model.AreaEmpty(pos1));
                Assert.AreEqual(preMoney, afterMoney + model.GetEntityIdentifier(TypeNameEnum.Road).PriceToPlace);
            }


            //Trying to 'overbuild' the road with other type
            InitLists();
            model.SelectForCurrentElement(TypeNameEnum.Bin);
            for (int i = 0; i < 10; i++)
            {
                Position pos1 = new Position(8 + i, 5);
                foreach (var enumerator in placeableInfrastructures)
                {
                    model.SelectForCurrentElement(enumerator);
                    Assert.IsFalse(model.AreaEmpty(pos1));
                    Assert.IsFalse(model.PlaceEntity(pos1, index - 1));
                }
            }

            //Trying the placement of of Cleaner and Mechanic alternating
            for (int i = 0; i < 10; i++)
            {
                Position pos1 = new Position(8 + i, 5);
                foreach (var enumerator in allStaff)
                {
                    model.SelectForCurrentElement(enumerator);
                    Assert.IsTrue(model.AreaEmpty(pos1));
                }

                model.SelectForCurrentElement(TypeNameEnum.Mechanic);
                if (i % 2 == 0)
                {
                    model.SelectForCurrentElement(TypeNameEnum.Cleaner);
                }

                Assert.IsTrue(model.PlaceEntity(pos1, index++));
            }

            int buildTime = model.GetEntityIdentifier(TypeNameEnum.Cleaner).TimeToBuild;
            if (buildTime <= 0)
            {
                buildTime = 1;
            }

            for (int i = 0; i < buildTime; i++)
            {
                model.TestTick();
            }

            Assert.IsNotNull(model.GetGameInfo().Infrastructures[CategoryEnum.Road]
                .Find(e => e.Type == TypeNameEnum.Road));


            //Building another set of roads for rotation check
            model.SelectForCurrentElement(TypeNameEnum.Road);
            for (int i = 0; i < 10; i++)
            {
                Position pos1 = new Position(8, 6 + i);
                var info = model.GetEntitiyInfoForPosition(pos1);
                Assert.IsNull(info);

                long preMoney = model.GetMoney();
                Assert.IsTrue(model.PlaceEntity(pos1, index++));
                long afterMoney = model.GetMoney();

                Assert.IsFalse(model.AreaEmpty(pos1));
                Assert.AreEqual(preMoney, afterMoney + model.GetEntityIdentifier(TypeNameEnum.Road).PriceToPlace);
            }


            //Rotate ellen?rz�se
            for (int i = 0; i < 10; i++)
            {
                foreach (var staff in allStaff)
                {
                    model.SelectForCurrentElement(staff);
                    var preRot = model.CurrentID.ConstructionInformation.RotationTimes;
                    model.RotateCurrentElement();
                    var afterRot = model.CurrentID.ConstructionInformation.RotationTimes;
                    Assert.AreNotEqual(preRot, afterRot);
                    model.RotateCurrentElement();
                    model.RotateCurrentElement();
                    model.RotateCurrentElement();
                    afterRot = model.CurrentID.ConstructionInformation.RotationTimes;
                    Assert.AreEqual(preRot, afterRot);

                    model.PlaceEntity(new Position(8, 6 + i), index++);
                    Assert.IsNotNull(model.GetGameInfo().Table[8, 6 + 1]);
                    Assert.AreEqual(model.GetGameInfo().Table[8, 6 + 1].Identity.ConstructionInformation.RotationTimes,
                        afterRot);

                }
            }

            for (int i = 0; i < buildTime; i++)
            {
                model.TestTick();
            }

            Assert.AreEqual(15, model.GetGameInfo().People[TypeNameEnum.Cleaner].Count);
            Assert.AreEqual(15, model.GetGameInfo().People[TypeNameEnum.Mechanic].Count);


        }

        [TestMethod] //Non-Trivial
        public void CleanerReplacement()
        {
            //Construction
            Model model = new Model(true, "EntityConfigForConstruction.json");
            model.NewGame();
            Assert.AreEqual(model.GetSize(), new Tuple<int, int>(45, 29));
            Assert.IsTrue(model.GetListOfPlaceable().Count > 0);
            Assert.IsTrue(model.GetMoney() > 0);
            var list = model.GetListOfPlaceable();
            Assert.IsTrue(list.Count > 0);

            //Placing roads
            model.SelectForCurrentElement(TypeNameEnum.Road);
            Assert.IsTrue(model.PlaceEntity(new Position(5, 5), 1));
            Assert.IsTrue(model.PlaceEntity(new Position(5, 6), 2));
            Assert.IsTrue(model.PlaceEntity(new Position(11, 6), 3));

            //Placing cleaner
            model.SelectForCurrentElement(TypeNameEnum.Cleaner);
            Assert.IsTrue(model.PlaceEntity(new Position(5, 5), 4));
            model.TestTick();
            var info = model.GetGameInfo();
            var cleaners = info.People[TypeNameEnum.Cleaner];
            Assert.AreEqual(cleaners.Count, 1);
            Assert.AreEqual(cleaners[0].position, new Position(5, 5));
            Assert.AreEqual(model.SelectForModify(new Position(5, 5)).Item2, TypeNameEnum.Cleaner);

        }

        private static List<Position> SlotsForEntity(TypeNameEnum entity, Model model)
        {
            var prev = model.CurrentID.Type;
            model.SelectForCurrentElement(entity);
            List<Position> list = new List<Position>();
            var info = model.GetGameInfo();
            for (int i = 0; i < info.SizeN; i++)
            {
                for (int j = 0; j < info.SizeM; j++)
                {
                    Position p = new Position(i, j);
                    if (model.AreaEmpty(p))
                    {
                        list.Add(p);
                    }
                }
            }

            model.SelectForCurrentElement(prev);

            return list;
        }

        [TestMethod] //Non-Trivial
        public void SelectForModifyTest()
        {
            Model.N = 45;
            Model.M = 29;
            Model.StartMoney = 10000000;
            //Construction
            Model model = new Model(true, "EntityConfigForConstruction.json");
            model.NewGame();
            Assert.AreEqual(model.GetSize(), new Tuple<int, int>(45, 29));
            Assert.IsTrue(model.GetListOfPlaceable().Count > 0);
            Assert.IsTrue(model.GetMoney() > 0);
            var list = model.GetListOfPlaceable();
            Assert.IsTrue(list.Count > 0);

            //Placing Entity
            int index = 0;
            List<Infrastructure> placedInfrastructures1 = new List<Infrastructure>();
            List<Infrastructure> placedInfrastructures2 = new List<Infrastructure>();

            InitLists();
            foreach (TypeNameEnum type in placeableInfrastructures)
            {

                Random rand = new Random();
                model.SelectForCurrentElement(type);
                var places1 = SlotsForEntity(type, model);
                Position pos1 = places1[rand.Next(0, places1.Count - 1)];

                CategoryEnum cat = model.GetEntityIdentifier(type).Category;
                if (cat == CategoryEnum.Game)
                {
                    model.SetEntryPriceForCurrentElement(2000);
                    model.SetMinCapacityForCurrentElement(40);
                }
                else if (cat == CategoryEnum.Restaurant)
                {
                    model.SetEntryPriceForCurrentElement(2000);
                    model.SetMinCapacityForCurrentElement(40);
                }

                Assert.IsTrue(model.PlaceEntity(pos1, index++));
                var entity = model.GetGameInfo().Table[pos1.X, pos1.Y];

                placedInfrastructures1.Add(entity);
                for (int i = 0; i < entity.Identity.TimeToBuild; i++)
                {
                    model.TestTick();
                }


                //2
                var places2 = SlotsForEntity(type, model);
                Position pos2 = places2[rand.Next(0, places2.Count - 1)];
                model.SelectForCurrentElement(type);
                if (cat == CategoryEnum.Game)
                {
                    model.SetEntryPriceForCurrentElement(1000);
                    model.SetMinCapacityForCurrentElement(50);
                }
                else if (cat == CategoryEnum.Restaurant)
                {
                    model.SetEntryPriceForCurrentElement(1000);
                    model.SetMinCapacityForCurrentElement(50);
                }

                Assert.IsTrue(model.PlaceEntity(pos2, index++));
                var entity2 = model.GetGameInfo().Table[pos2.X, pos2.Y];

                placedInfrastructures2.Add(entity2);
                for (int i = 0; i < entity2.Identity.TimeToBuild; i++)
                {
                    model.TestTick();
                }

            }

            int ind = 0;
            foreach (var infra in placedInfrastructures1)
            {
                if (infra is Game)
                {
                    model.SelectForModify(infra.position);
                    model.SelectForModify(infra.position);
                    int prevPrice = model.GetEntryPriceForCurrentElement();
                    int prevCap = model.GetMinCapacityForCurrentElement();
                    int nextPrice = prevPrice + 1500;
                    int nextCap = 90;
                   


                    model.SetEntryPriceForCurrentElement(nextPrice);
                    model.SetMinCapacityForCurrentElement(nextCap);
                    Assert.AreEqual(nextCap, model.GetMinCapacityForCurrentElement(), 5);
                    Assert.AreEqual(nextPrice, model.GetEntryPriceForCurrentElement(), 5);

                    model.SelectForModify(placedInfrastructures2[ind].position);
                    Assert.AreNotEqual(nextPrice, model.GetEntryPriceForCurrentElement());
                    Assert.AreNotEqual(nextCap, model.GetMinCapacityForCurrentElement());
                }

                else if (infra is Restaurant)
                {
                    model.SelectForModify(infra.position);
                    model.SelectForModify(infra.position);
                    int prevPrice = model.GetEntryPriceForCurrentElement();
                    int prevCap = model.GetMinCapacityForCurrentElement();
                    int nextPrice = prevPrice + 1500;
                    int nextCap = 90;

                    Debug.WriteLine(nextCap);
                    
                    model.SetEntryPriceForCurrentElement(nextPrice);
                    model.SetMinCapacityForCurrentElement(nextCap);
                    nextCap = Convert.ToInt32(Math.Round(
                        ((float) infra.Identity.ConstructionInformation.MinimumCapacity / infra.Identity.Capacity) *
                        100.0));
                    Assert.AreEqual(nextCap, model.GetMinCapacityForCurrentElement());
                    Assert.AreEqual(nextPrice, model.GetEntryPriceForCurrentElement());

                    model.SelectForModify(placedInfrastructures2[ind].position);
                    Assert.AreNotEqual(nextPrice, model.GetEntryPriceForCurrentElement());
                    Assert.AreNotEqual(nextCap, model.GetMinCapacityForCurrentElement());
                }

                ind++;

            }


        }

        [TestMethod] //Non-Trivial
        public void TestGraph()
        {
            Model.N = 45;
            Model.M = 29;
            //Construction
            Model model = new Model(true, "EntityConfigForConstruction.json");
            model.NewGame();
            Assert.AreEqual(model.GetSize(), new Tuple<int, int>(45, 29));
            Assert.IsTrue(model.GetListOfPlaceable().Count > 0);
            Assert.IsTrue(model.GetMoney() > 0);
            var list = model.GetListOfPlaceable();
            Assert.IsTrue(list.Count > 0);

            model.SelectForCurrentElement(TypeNameEnum.Road);
            Assert.IsTrue(model.PlaceEntity(new Position(3, 3), 1));
            Assert.IsTrue(model.PlaceEntity(new Position(3, 4), 2));
            Assert.IsTrue(model.PlaceEntity(new Position(3, 5), 3));
            Assert.IsTrue(model.PlaceEntity(new Position(3, 6), 4));
            Assert.IsTrue(model.PlaceEntity(new Position(3, 7), 5));
            Assert.IsTrue(model.PlaceEntity(new Position(3, 8), 6));
            Assert.IsTrue(model.PlaceEntity(new Position(4, 8), 7));
            Assert.IsTrue(model.PlaceEntity(new Position(5, 8), 8));
            Assert.IsTrue(model.PlaceEntity(new Position(6, 8), 9));
            Assert.IsTrue(model.PlaceEntity(new Position(7, 8), 10));
            Assert.IsTrue(model.PlaceEntity(new Position(8, 8), 11));
            Assert.IsTrue(model.PlaceEntity(new Position(9, 8), 12));

            model.SelectForCurrentElement(TypeNameEnum.CarouselGame);
            model.RotateCurrentElement();
            model.RotateCurrentElement();
            Assert.IsTrue(model.PlaceEntity(new Position(6, 9), 13));

            Assert.IsTrue(model.PlaceEntity(new Position(20, 20), 14));


            var dijkstra = new Satsuma.Dijkstra(model.GetGameInfo().Graph, e => 1.0, DijkstraMode.Sum);
            dijkstra.AddSource((model.GetGameInfo().Table[3, 3] as Road).Node);
            dijkstra.RunUntilFixed((model.GetGameInfo().Table[9, 8] as Road).Node);
            double d = dijkstra.GetDistance((model.GetGameInfo().Table[9, 8] as Road).Node);
            Assert.AreEqual(d, 11);


            dijkstra = new Satsuma.Dijkstra(model.GetGameInfo().Graph, e => 1.0, DijkstraMode.Sum);
            dijkstra.AddSource((model.GetGameInfo().Table[3, 3] as Road).Node);
            dijkstra.RunUntilFixed((model.GetGameInfo().Table[6, 9] as CarouselGame).Node);
            d = dijkstra.GetDistance((model.GetGameInfo().Table[6, 9] as CarouselGame).Node);
            Assert.AreEqual(d, 10);

            dijkstra = new Satsuma.Dijkstra(model.GetGameInfo().Graph, e => 1.0, DijkstraMode.Sum);
            dijkstra.AddSource((model.GetGameInfo().Table[3, 3] as Road).Node);
            dijkstra.RunUntilFixed((model.GetGameInfo().Table[20, 20] as CarouselGame).Node);
            d = dijkstra.GetDistance((model.GetGameInfo().Table[20, 20] as CarouselGame).Node);
            Assert.IsTrue(Double.IsInfinity(d));

        }

        private static Position NeighbouringField(Position field, Model model)
        {

            Random random = new Random();
            List<Position> positions = new List<Position>
            {
                new Position(field.X - 1, field.Y),
                new Position(field.X + 1, field.Y),
                new Position(field.X, field.Y - 1),
                new Position(field.X, field.Y + 1),
            };

            List<Position> approved = new List<Position>();

            foreach (Position position in positions)
            {
                if (model.AreaEmpty(position))
                {
                    approved.Add(position);
                }
            }

            if (approved.Count == 0)
            {
                return null;
            }
            else
            {
                return approved[random.Next(0, approved.Count)];
            }



        }

        private void SimulateConstruction(int n = 45, int m = 29, int maxTick = 1000, int buildAmount = 1, double maxBuildRatio = 0.6)
        {
            int TICK = 0;
            Assert.IsTrue(buildAmount >= 1);

            //Construction
            Model.N = n;
            Model.M = m;
            Model model = new Model(true, "EntityConfigForConstruction.json");
            model.NewGame();
            Assert.AreEqual(model.GetSize(), new Tuple<int, int>(n, m));
            Assert.IsTrue(model.GetListOfPlaceable().Count > 0);
            Assert.IsTrue(model.GetMoney() > 0);
            var list = model.GetListOfPlaceable();
            Assert.IsTrue(list.Count > 0);

            //Building a map
            InitLists();
            int ind = 0;
            int area = 0;
            var FirstBuildingSession = new List<TypeNameEnum>(placeableInfrastructures);
            for (int i = 0; i < buildAmount - 1; i++)
            {
                foreach (TypeNameEnum nameEnum in placeableInfrastructures)
                {
                    FirstBuildingSession.Add(nameEnum);
                }
            }

            while (TICK < maxTick && FirstBuildingSession.Count > 0 && area < (n * m) * maxBuildRatio)
            {
                Random rand = new Random();
                TypeNameEnum type = FirstBuildingSession[rand.Next(0, FirstBuildingSession.Count - 1)];
                FirstBuildingSession.Remove(type);
                Debug.WriteLine(area + " - " + (n * m));
                var places = SlotsForEntity(type, model);
                Position place = places[rand.Next(0, places.Count - 1)];
                model.SelectForCurrentElement(type);
                Assert.IsTrue(model.PlaceEntity(place, ind++));
                Assert.IsNotNull(model.GetGameInfo().Table[place.X, place.Y]);
                area += model.GetEntityIdentifier(type).SizeM * model.GetEntityIdentifier(type).SizeN;
                model.TestTick();
                TICK++;
            }

            Assert.IsTrue(TICK < maxTick);

            // Build road
            List<Position> roads = new List<Position>();
            model.SelectForCurrentElement(TypeNameEnum.Road);
            var roadPlaces = SlotsForEntity(TypeNameEnum.Road, model);
            bool possibleToPlace = roadPlaces.Count > 0;
            Assert.IsTrue(possibleToPlace);
            Position nextPos = roadPlaces[new Random().Next(0, roadPlaces.Count - 1)];

            int roadsPlaced = 0;
            while (nextPos is not null)
            {
                Debug.WriteLine(roadsPlaced);
                model.SelectForCurrentElement(TypeNameEnum.Road);
                Assert.IsTrue(model.AreaEmpty(nextPos));
                Assert.IsTrue(model.PlaceEntity(nextPos, ind++));
                roadsPlaced++;
                roads.Add(nextPos);
                nextPos = NeighbouringField(nextPos, model);
                model.TestTick();
            }

            //Place staff
            Assert.IsTrue(roadsPlaced > 0);
            int numberOfCleaners = new Random().Next(1, Math.Max(roadsPlaced / 4, 1));
            int numberOfMechanic = new Random().Next(1, Math.Max(roadsPlaced / 4, 1));

            List<Position> cleanerPos = new List<Position>();

            model.SelectForCurrentElement(TypeNameEnum.Cleaner);
            for (int i = 0; i < numberOfCleaners; i++)
            {

                Position pos = roads[new Random().Next(0, roads.Count - 1)];
                while (!model.AreaEmpty(pos))
                {
                    pos = roads[new Random().Next(0, roads.Count - 1)];
                }

                Assert.IsTrue(model.PlaceEntity(pos, ind++));
                cleanerPos.Add(pos);
                roads.Remove(pos);
                model.TestTick();
            }

            model.SelectForCurrentElement(TypeNameEnum.Mechanic);

            for (int i = 0; i < numberOfMechanic; i++)
            {
                Position pos = roads[new Random().Next(0, roads.Count - 1)];
                while (!model.AreaEmpty(pos))
                {
                    pos = roads[new Random().Next(0, roads.Count - 1)];
                }

                Assert.IsTrue(model.PlaceEntity(pos, ind++));
                cleanerPos.Add(pos);
                roads.Remove(pos);
                model.TestTick();
            }
        }

        //This method builds a random park
        [TestMethod] //Non-Trivial - This method does not test Graph Router Searching Algorithms, because the algo. is good as proven before
        public void SimulatedBuildingCase()
        {
            Model.N = 45;
            Model.M = 29;
            Model.StartMoney = 10000000;
            SimulateConstruction();
        }

        //This method builds a LARGE random park
        [TestMethod] //Non-Trivial - This method does not test Graph Router Searching Algorithms, because the algo. is good as proven before
        public void SimulatedBuildingCaseMassive()
        {
            Model.N = 45;
            Model.M = 29;
            Model.StartMoney = 100000000;
            SimulateConstruction(1000, 1000, 10000, 20);
        }

        [TestMethod] //Non-Trivial
        public void GameOver()
        {
            Model.N = 45;
            Model.M = 29;
            Model.StartMoney = 10000000;
            //Construction
            Model model = new Model(test: true);
            model.NewGame();
            Assert.IsTrue(model.GetListOfPlaceable().Count > 0);
            Assert.IsTrue(model.GetMoney() > 0);
            var list = model.GetListOfPlaceable();
            Assert.IsTrue(list.Count > 0);

            //Building a map
            InitLists();
            int ind = 0;
            var BuildingSession = new List<TypeNameEnum>(placeableInfrastructures);
            
            while (BuildingSession.Count > 0 )
            {
                Random rand = new Random();
                TypeNameEnum type = BuildingSession[rand.Next(0, BuildingSession.Count - 1)];
                Debug.WriteLine(type.ToString());
                BuildingSession.Remove(type);
                var places = SlotsForEntity(type, model);
                Position place = places[rand.Next(0, places.Count - 1)];
                model.SelectForCurrentElement(type);
                Assert.IsTrue(model.PlaceEntity(place, ind++));
                Assert.IsNotNull(model.GetGameInfo().Table[place.X, place.Y]);
                model.TestTick();
            }
            
            model.StartGame(1000);
            while (model.GetMoney() >= -10)
            {
                Debug.WriteLine(model.GetMoney());
                model.TestTick();
            }
            
        }
    }

}
