﻿using System;
using System.Collections.Generic;
using System.Text;
using System.Collections.ObjectModel;
using CSIP_42.Model;
using System.Diagnostics;
using System.Windows;
using System.Windows.Input;
using System.Linq;
using CSIP_42.Model.Enumerators;

namespace CSIP_42.ViewModel
{
    public class MainViewModel : ViewModelBase
    {
        #region Fields
        private readonly Model.Model _model;

        private string _currentGamePrice;
        private string _currentGameMinCapacity;
        private string _currentServicePrice;
        private string _parkEntryPrice;

        private double _canvasMouseX;
        private double _canvasMouseY;
        private double _canvasWidth;
        private double _canvasHeight;

        private double _renderTableHeight;
        private double _renderTableWidth;
        private double _xOffset;
        private double _yOffset;
        private int _tableMouseXWithOffset;
        private int _tableMouseYWithOffset;

        private string _itemExtraDetailsToolTipText;
        private string _itemExtraDetailsToolTipVisibility;

        private string _statusBar;

        private ListElem _selectedListElem;
        private ObservableCollection<ListElem> _filteredListElems;
        private ObservableCollection<PlacedItem> _placedItems;
        private bool _filteringAndListingIsEnabled;

        #endregion Fields


        #region Properties - Elem filtering, listing
        /// <summary>
        /// Contains every placeable item that can be listed.
        /// </summary>
        public List<ListElem> AllListElems { get; private set; }
        /// <summary>
        /// Contains the filters of the items' list.
        /// </summary>
        public ObservableCollection<Filter> Filters { get; private set; }
        /// <summary>
        /// Contains those items that are in the checked categories.
        /// </summary>
        public ObservableCollection<ListElem> FilteredListElems
        {
            get { return _filteredListElems; }
            private set
            {
                _filteredListElems = value;
                OnPropertyChanged();
            }
        }
        /// <summary>
        /// The elem from the List which is selected. If none selected then null.
        /// </summary>
        public ListElem SelectedListElem
        {
            get { return _selectedListElem; }
            set
            {
                if (_selectedListElem != value)
                {
                    _selectedListElem = value;
                    if (value != null)
                    {
                        PlacedItems[0].WidthOnTable = _selectedListElem.Width;
                        PlacedItems[0].HeightOnTable = _selectedListElem.Height;
                        PlacedItems[0].Rotation = 0;
                        _model.SelectForCurrentElement(_selectedListElem.Type);
                    }
                    OnPropertyChanged();
                }
            }
        }
        public bool FilteringAndListingIsEnabled
        {
            get { return _filteringAndListingIsEnabled; }
            set
            {
                _filteringAndListingIsEnabled = value;
                OnPropertyChanged();
            }
        }
        #endregion Properties - Elem filtering, listing

        #region Properties - Placed items' datas
        /// <summary>
        /// Contains every item which is placed on the canvas
        /// </summary>
        public ObservableCollection<PlacedItem> PlacedItems
        {
            get { return _placedItems; }
            private set
            {
                _placedItems = value;
                OnPropertyChanged();
            }
        }
        /// <summary>
        /// Contains the extra details (if it has any) about the elem on which the cursor is.
        /// </summary>
        public string ItemExtraDetailsToolTipText
        {
            get { return _itemExtraDetailsToolTipText; }
            set
            {
                if (_itemExtraDetailsToolTipText != value)
                {
                    _itemExtraDetailsToolTipText = value;
                    ItemExtraDetailsToolTipVisibility = _itemExtraDetailsToolTipText.Length > 0 ? "Visible": "Hidden";
                    OnPropertyChanged();
                }
            }
        }
        /// <summary>
        /// Gives the visibility of the tooltip of extra details.
        /// </summary>
        public string ItemExtraDetailsToolTipVisibility
        {
            get { return _itemExtraDetailsToolTipVisibility; }
            set
            {
                if (value != _itemExtraDetailsToolTipVisibility)
                {
                    _itemExtraDetailsToolTipVisibility = value;
                    OnPropertyChanged();
                }
            }
        }
        #endregion Properties - Placed items' datas

        #region Properties - Currently set game/park/service 
        /// <summary>
        /// The min capacity of the currently placed/modified game which the user types in. Not converted, not validated.
        /// </summary>
        public string CurrentGameMinCapacity
        {
            get { return _currentGameMinCapacity; }
            set
            {
                _currentGameMinCapacity = value;
                OnPropertyChanged();
            }
        }
        /// <summary>
        /// The price of the currently placed/modified game which the user types in. Not converted, not validated.
        /// </summary>
        public string CurrentGamePrice
        {
            get { return _currentGamePrice; }
            set
            {
                _currentGamePrice = value;
                OnPropertyChanged();
            }
        }
        /// <summary>
        /// The price of the currently placed/modified service which the user types in. Not converted, not validated.
        /// </summary>
        public string CurrentServicePrice
        {
            get { return _currentServicePrice; }
            set
            {
                _currentServicePrice = value;
                OnPropertyChanged();
            }
        }
        /// <summary>
        /// The park's entry price. Not converted, not validated.
        /// </summary>
        public string ParkEntryPrice
        {
            get { return _parkEntryPrice; }
            set
            {
                _parkEntryPrice = value;
                OnPropertyChanged();
            }
        }
        /// <summary>
        /// The min capacity of the currently placed/modified game which the user types in. Converted, validated.
        /// </summary>
        private int IntCurrentGameMinCapacity { get; set; }
        /// <summary>
        /// The price of the currently placed/modified service which the user types in. Converted, validated.
        /// </summary>
        private int IntCurrentGamePrice { get; set; }
        /// <summary>
        /// The price of the currently placed/modified service which the user types in. Converted, validated.
        /// </summary>
        private int IntCurrentServicePrice { get; set; }
        /// <summary>
        /// The park's entry price. Converted, validated.
        /// </summary>
        private int IntParkEntryPrice { get; set; }
        #endregion Properties - Currently set game/park/service 

        #region Properties - Statusbar
        public long GamerMoney { get; private set; }
        public string GameState { get; private set; }
        public string StatusBar 
        {
            get { return _statusBar; }
            private set 
            {
                if (value != _statusBar)
                {
                    _statusBar = value;
                    OnPropertyChanged();
                }
            }
        }
        #endregion Properties - Statusbar

        #region Properties - Game table
        public int TableHeight { get; private set; }
        public int TableWidth { get; private set; }
        public double RenderedTableHeight
        {
            get { return _renderTableHeight; }
            private set
            {
                if(_renderTableHeight != value)
                    _renderTableHeight = value;
            }
        }
        public double RenderedTableWidth
        {
            get { return _renderTableWidth; }
            private set
            {
                if (_renderTableWidth != value)
                {
                    _renderTableWidth = value;
                    RenderedTableHeight = (_renderTableWidth * CanvasHeight) / CanvasWidth;
                    PanelWidthHeightUnit = CanvasWidth / _renderTableWidth;
                }
            }
        }
        /// <summary>
        /// The x coordinate of the panel on which the cursor is. Contains the offset needed for the moving.
        /// The mousetracker updates it always.
        /// </summary>
        private int TableMouseXWithOffset
        {
            get { return _tableMouseXWithOffset; }
            set
            {
                if (value != _tableMouseXWithOffset) // ha különbözik akkor átlépett az egér egy másik panelbe
                {
                    _tableMouseXWithOffset = value;
                    if (SelectedListElem != null)
                        DrawHover(); // frissítjük a hovert is
                    else // ha nincs kiválasztott tehát nem akar lehelyezni
                        if (CanvasLoaded) // és a canvas már be van töltve
                        UpdateDetails(); // akkor frissítsük a tooltipet
                }
            }
        }
        /// <summary>
        /// The y coordinate of the panel on which the cursor is. Contains the offset needed for the moving.
        /// The mousetracker updates it always.
        /// </summary>
        private int TableMouseYWithOffset
        {
            get { return _tableMouseYWithOffset; }
            set
            {
                if (value != _tableMouseYWithOffset) // ha különbözik akkor átlépett az egér egy másik panelbe
                {
                    _tableMouseYWithOffset = value;
                    if (SelectedListElem != null)
                        DrawHover();// frissítjük a hovert is
                    else // ha nincs kiválasztott tehát nem akar lehelyezni
                        if (CanvasLoaded)// és a canvas már be van töltve
                        UpdateDetails();// akkor frissítsük a tooltipet
                }
            }
        }
        #endregion Properties - Game table

        #region Properties - Canvas
        private bool CanvasLoaded { get; set; }
        public double CanvasWidth
        {
            get { return _canvasWidth; }
            set
            {
                if (value != _canvasWidth)
                    _canvasWidth = value;
            }
        }
        public double CanvasHeight
        {
            get { return _canvasHeight; }
            set
            {
                if (value != _canvasHeight)
                    _canvasHeight = value;
            }
        }
        public double CanvasMouseX 
        {
            get { return _canvasMouseX; }
            set
            {
                _canvasMouseX = value;
                int xCoordWithOffset = (int)((value + XOffset) / (CanvasWidth / RenderedTableWidth)); // akkor kiszámoljuk hogy melyik panel offsettel
                TableMouseXWithOffset = xCoordWithOffset;// ekkor frissítjük
            }
        }
        public double CanvasMouseY
        {
            get { return _canvasMouseY; }
            set
            {
                _canvasMouseY = value;

                int yCoordWithOffset = (int)((value + YOffset) / (CanvasHeight / RenderedTableHeight));// akkor kiszámoljuk hogy melyik panel offsettel            
                TableMouseYWithOffset = yCoordWithOffset;// ekkor frissítjük          
            }
        }
        /// <summary>
        /// The x coordinate of the offset that is needed for moving the map.
        /// </summary>
        private double XOffset
        {
            get { return _xOffset; }
            set
            {
                _xOffset = value;
                // ha változott az offset akkor a panelkoordináta offsettel is változhatott
                int xCoordWithOffset = (int)((CanvasMouseX + _xOffset) / (CanvasWidth / RenderedTableWidth));
                if (xCoordWithOffset != TableMouseXWithOffset) // ha változott 
                {
                    TableMouseXWithOffset = xCoordWithOffset; // akkor frissítjük
                }
            }
        }
        /// <summary>
        /// The y coordinate of the offset that is needed for moving the map.
        /// </summary>
        private double YOffset
        {
            get { return _yOffset; }
            set
            {
                _yOffset = value;
                // ha változott az offset akkor a panelkoordináta offsettel is változhatott
                int yCoordWithOffset = (int)((CanvasMouseY + _yOffset) / (CanvasHeight / RenderedTableHeight));
                if (yCoordWithOffset != TableMouseYWithOffset)// ha változott 
                {
                    TableMouseYWithOffset = yCoordWithOffset;// akkor frissítjük
                }
            }
        }
        #endregion Properties - Canvas

        #region Properties - Others
        private double PanelWidthHeightUnit { get; set; }
        private double ShiftUnit { get; set; }

        private bool IsReplacingHappening { get; set; }
        #endregion Properties - Others



        #region Events - MainWindow's events
        public event EventHandler OpenInfoWindowEvent;
        public event EventHandler OpenGameSettingsDialogWindowEvent;
        public event EventHandler OpenServiceSettingsDialogWindowEvent;
        public event EventHandler OpenParkSettingsDialogWindowEvent;
        public event EventHandler ExitGame;
        public event EventHandler<long> GameOverEvent;
        public event EventHandler<String> ErrorEvent;
        #endregion Events - MainWindow's events

        #region Events - Extra windows' events
        public event EventHandler CloseGameSettingsDialogWindowEvent;
        public event EventHandler CloseServiceSettingsDialogWindowEvent;
        public event EventHandler CloseParkSettingsDialogWindowEvent;
        #endregion Events - Extra windows' events


        #region Commands - MainWindow's commands 
        public DelegateCommand NewGameCommand { get; private set; }
        public DelegateCommand OpenParkSettingsDialogWindowCommand { get; private set; }
        public DelegateCommand OpenInfoWindowCommand { get; private set; }
        public DelegateCommand ExitCommand { get; private set; }

        public DelegateCommand SelectAllFilterCommand { get; private set; }
        public DelegateCommand SelectNoneFilterCommand { get; private set; }
        #endregion Commands - MainWindow's commands

        #region Commands - Extra window's commands
        public DelegateCommand GameSettingsDialogWindowOkButtonCommand { get; private set; }
        public DelegateCommand ServiceSettingsDialogWindowOkButtonCommand { get; private set; }
        public DelegateCommand ParkSettingsDialogWindowOkButtonCommand { get; private set; }
        #endregion Commands - Extra window's commands


        #region Model event handlers
        /// <summary>
        /// <c>Model</c> sends in a <c>List</c> the changes of the elements, 
        /// giving their indices that it recieves after every item placing.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="listOfWhatToChange"></param>
        private void SimulationEventHandler(object sender, List<SimulationEventArgs> listOfWhatToChange)
        {
            foreach (var elem in listOfWhatToChange)
                PlacedItems[elem.index].ImageSource = "Images/" + elem.feeling.ToString() + elem.who.ToString() + ".png";
        }

        private void StatusBarChangedEventHandler(object sender, Tuple<long,int> tuple)
        {
            GamerMoney = tuple.Item1;
            if(tuple.Item2 == -1)
            {
                StatusBar = "Money: " + String.Format("{0:n0}", GamerMoney) + " Ft | Park is: " + GameState;
            }
            else
            {
                StatusBar = "Money: " + String.Format("{0:n0}", GamerMoney) + " Ft | Park is: " + GameState +
                    " | Percentage of last 10 people that came in: " + tuple.Item2 + "%";

            }
            UpdateAvailableListElems();
        }


        private void GameOverEventHandler(object sender, long elapsedTime)
        {
            GameOverEvent?.Invoke(this, elapsedTime);
        }
        #endregion Model event handlers

        #region View event handlers
        private void FilterChangedEventHandler(object sender, EventArgs e)
        {
            Filter senderFilter = sender as Filter;
            if (senderFilter.IsChecked == true) // bepipálták
            {
                foreach (ListElem elem in AllListElems)
                {
                    if (elem.Category == senderFilter.Category)
                        FilteredListElems.Insert(0, elem); // mert szeretnénk hogy elöl jelenjen meg a legutoljára bepipált
                }
            }
            else // kipipálták
            {
                ObservableCollection<ListElem> newFilteredListElems = new();
                foreach (ListElem elem in FilteredListElems)
                {
                    if (elem.Category != senderFilter.Category)
                        newFilteredListElems.Add(elem);
                }
                FilteredListElems = newFilteredListElems;
            }
        }

        /// <summary>
        /// Sets <c>CanvasWidth</c>, <c>CanvasHeight</c>, <c>RenderTableWidth</c>, <c>RenderTableHeigth</c>, <c>ShiftUnit</c>, and the offsets,
        /// builds the starter table, and sets <c>CanvasLoaded true</c>.
        /// </summary>
        /// <Possible_development>ne a window-n legyen a gombnyomásfigyelés hanem a canvason, 
        /// de úgy a fókusszal adódnak gondok</Possible_development>
        /// <Possible_development>Az offset beállítása ne legyen beégetve, a modelltől jöjjön,
        /// hogy az éppne hova teszi a főkaput</Possible_development>
        public void CanvasLoadedEventHandler(object sender, EventArgs e)
        {       
            CanvasWidth = (sender as System.Windows.Controls.Canvas).ActualWidth;
            CanvasHeight = (sender as System.Windows.Controls.Canvas).ActualHeight;

            RenderedTableWidth = 25; // a setterében beállítja hozzá kiszámolva a RenderTableHeight-ot
            ShiftUnit = 30;

            // bal alulra középre tegyük a kezdőpozíciót, itt lesz a főkapu
            XOffset = ((TableWidth - RenderedTableWidth) * PanelWidthHeightUnit) / 2;
            YOffset = (TableHeight - RenderedTableHeight) * PanelWidthHeightUnit;

            BuildTable();
            CanvasLoaded = true;
        }

        /// <summary>
        /// Handles the mousewheel as zooming in or out depending on the <c>MouseWheelEventArgs</c> parameter's <c>Delta</c> property.
        /// </summary>
        public void CanvasMouseWheelEventHandler(object sender, MouseWheelEventArgs e)
        {
            ZoomMap(e.Delta);
        }

        // Ez akár kitörölhető, már nem lehet átméretezni, fullscreenes az alkalmazás, de tesztelni jó vele, később akár hasznos lehet
        public void CanvasResizedEventHandler(object sender, SizeChangedEventArgs e)
        {
            CanvasWidth = e.NewSize.Width;
            CanvasHeight = e.NewSize.Height;
        }
      
        /// <summary>
        /// Puts hover's visibility hidden.
        /// </summary>
        public void CanvasMouseLeaveEventHandler(object sender, MouseEventArgs e)
        {
            TableMouseXWithOffset = -1; // ez azért kell mert ha kivisszük az egeret, majd ugyanabban a kockába visszavisszük, akkor nem jelenne meg a hover, hiszen nem változott
            TableMouseYWithOffset = -1; // ez azért kell mert ha kivisszük az egeret, majd ugyanabban a kockába visszavisszük, akkor nem jelenne meg a hover, hiszen nem változott           
            PlacedItems[0].Visibility = "Hidden"; // Ha az egér elhagyja a canvas-t akkor nem kell hover      
        }

        /// <summary>
        /// Calls the needed function for placing item or modifying it's datas.
        /// </summary>
        public void CanvasClickedEventHandler(object sender, MouseEventArgs e)
        {
            //azért nem használom a folyamatosan updatelt panel változókat, mert elképzelhető, (biztosan?)
            // hogy olyan gyorsan ráncigálták az egeret hogy még nem frissült.
            // Nem tudom, hogy van többszálúsítva a dolog. De a biztonság kedvéért újból kiszámolom.
            var pos = e.GetPosition(sender as IInputElement);
            int yCoordWithOffset = (int)((pos.Y + YOffset) / (CanvasHeight / RenderedTableHeight));
            int xCoordWithOffset = (int)((pos.X + XOffset) / (CanvasWidth / RenderedTableWidth));

            if (SelectedListElem != null && GamerMoney >= SelectedListElem.Price)
            {
                if (SelectedListElem.Category == CategoryEnum.Game)
                    PlaceGame(yCoordWithOffset, xCoordWithOffset);
                else if (SelectedListElem.Category == CategoryEnum.Toalett || SelectedListElem.Category == CategoryEnum.Restaurant)
                    PlaceService(yCoordWithOffset, xCoordWithOffset);
                else if (SelectedListElem.Category == CategoryEnum.Staff)
                    PlaceStaff(yCoordWithOffset, xCoordWithOffset);
                else
                    PlaceOther(yCoordWithOffset, xCoordWithOffset);
            }
            else // ha áthelyezés, vagy adatok újabb beállítása van
            {
                try // a _model.SelectForModify dobhat kivételt, pl ha nincs semmi az adott pozíción
                {
                    Tuple<CategoryEnum, TypeNameEnum> type = _model.SelectForModify(new Position(xCoordWithOffset, yCoordWithOffset));
                    if (type.Item1 == CategoryEnum.Game)
                        ModifyGameDatas();
                    else if (type.Item1 == CategoryEnum.Toalett || type.Item1 == CategoryEnum.Restaurant)
                        ModifyServiceDatas();
                    else if (type.Item2 == TypeNameEnum.MainGate)
                        ModifyMainGateDatas();
                    else if (type.Item2 == TypeNameEnum.Cleaner)
                        ReplaceCleaner(yCoordWithOffset, xCoordWithOffset);
                }
                catch (InvalidOperationException exc)// ekkor valami nagy baj történt
                {
                    throw new InvalidOperationException(exc.Message); 
                }
                catch (ArgumentException) {}
            }
        }

        /// <summary>
        /// Handles every keypress by calling the needed functions depending on the pressed key.
        /// </summary>
        public void PreviewKeyDownEventHandler(object sender, KeyEventArgs e)
        {
            if (e.Key == Key.W || e.Key == Key.S || e.Key == Key.A || e.Key == Key.D)
                MoveMap(e.Key);
            else if (e.Key == Key.R)
                RotateSelectedListElem();
            else if (e.Key == Key.Escape)
                UnselectSelectedListElem();
        }

        /// <summary>
        /// Updates <c>CurrentGamePrice</c> by the value written in the setting window.
        /// </summary>
        public void CurrentGamePriceTextBoxTextChangedEventHandler(object sender, System.Windows.Controls.TextChangedEventArgs e)
        {
            CurrentGamePrice = (sender as System.Windows.Controls.TextBox).Text;
        }

        /// <summary>
        /// Updates <c>CurrentGameMinCapacity</c> by the value written in the setting window.
        /// </summary>
        public void CurrentGameMinCapacityTextBoxTextChangedEventHandler(object sender, System.Windows.Controls.TextChangedEventArgs e)
        {
            CurrentGameMinCapacity = (sender as System.Windows.Controls.TextBox).Text;
        }

        /// <summary>
        /// Updates <c>ParkEntryPrice</c> by the value written in the setting window.
        /// </summary>
        public void ParkEntryPriceTextBoxTextChangedEventHandler(object sender, System.Windows.Controls.TextChangedEventArgs e)
        {
            ParkEntryPrice = (sender as System.Windows.Controls.TextBox).Text;
        } 
        #endregion View event handlers


        #region Constructor
        public MainViewModel(Model.Model model)
        {
            _model = model;
            _model.StatusBarChangedEvent += StatusBarChangedEventHandler;
            _model.SimulationEvent += SimulationEventHandler;
            _model.GameOverEvent += GameOverEventHandler;

            CanvasLoaded = false;

            TableWidth = _model.GetSize().Item1; //sizeN - X tengely
            TableHeight = _model.GetSize().Item2; //sizeM - Y tengely
            PlacedItems = new ObservableCollection<PlacedItem>
            {
                new PlacedItem { IsHover = true } // az első kitüntett lesz, ez lesz a hover
            };
            GameState = "Closed";
            GamerMoney = _model.GetMoney();
            StatusBar = "Money: " + GamerMoney.ToString() + " Ft | Park is: " + GameState;
            IsReplacingHappening = false;
            IntCurrentGameMinCapacity = -1;
            IntCurrentGamePrice = -1;
            IntCurrentServicePrice = -1;
            IntParkEntryPrice = -1;
            ItemExtraDetailsToolTipVisibility = "Hidden";
            ItemExtraDetailsToolTipText = "";
            FilteringAndListingIsEnabled = true;
            SelectedListElem = null;

            AllListElems = new List<ListElem>();
            InitializeAllListElems(_model.GetListOfPlaceable());
            FilteredListElems = new ObservableCollection<ListElem>();

            Filters = new ObservableCollection<Filter>
            {
                new Filter { Name = "Games", IsChecked = false, Category = CategoryEnum.Game },
                new Filter { Name = "Roads", IsChecked = false, Category = CategoryEnum.Road },
                new Filter { Name = "Restaurants", IsChecked = false, Category = CategoryEnum.Restaurant },
                new Filter { Name = "Plant", IsChecked = false, Category = CategoryEnum.Plant },
                new Filter { Name = "Toalett", IsChecked = false, Category = CategoryEnum.Toalett },
                new Filter { Name = "Staff", IsChecked = false, Category = CategoryEnum.Staff },
                new Filter { Name = "Bin", IsChecked = false, Category = CategoryEnum.Bin }
            };
            foreach (Filter filter in Filters)
                filter.FilterChanged += new EventHandler(FilterChangedEventHandler);

            SelectAllFilterCommand = new DelegateCommand(par => SelectAllFilter());
            SelectNoneFilterCommand = new DelegateCommand(par => SelectNoneFilter());
            NewGameCommand = new DelegateCommand(par => NewGame());
            OpenInfoWindowCommand = new DelegateCommand(par => OpenInfoWindowEvent?.Invoke(this, EventArgs.Empty));
            OpenParkSettingsDialogWindowCommand = new DelegateCommand(par => OpenPark());        
            GameSettingsDialogWindowOkButtonCommand = new DelegateCommand(par => CloseGameSettingsDialogWindow());
            ServiceSettingsDialogWindowOkButtonCommand = new DelegateCommand(par => CloseServicePriceDialogWindow());
            ParkSettingsDialogWindowOkButtonCommand = new DelegateCommand(par => CloseParkSettingsDialogWindow());
            ExitCommand = new DelegateCommand(par => ExitGame?.Invoke(this, EventArgs.Empty));
            
        }


        #endregion Constructor


        #region Methods - Filtering, listing
        private void InitializeAllListElems(List<EntityIdentifier> list)
        {
            foreach (var elem in list)
            {
                AllListElems.Add(new ListElem()
                {
                    Type = elem.Type,
                    Category = elem.Category,
                    Price = elem.PriceToPlace,
                    HasAlwaysCost = elem.AlwaysCost != -1,
                    HasSingleCost = elem.SingleCost != -1,
                    AlwaysCost = "   Always cost: " + String.Format("{0:n0}", elem.AlwaysCost) + " Ft",
                    SingleCost = "   Single cost: " + String.Format("{0:n0}", elem.SingleCost) + " Ft",
                    Name = elem.DisplayInformationEnglish.DisplayName + " (" + String.Format("{0:n0}", elem.PriceToPlace) + " Ft)",                  
                    IsAvailable = GamerMoney >= elem.PriceToPlace,
                    Height = elem.SizeM,
                    Width = elem.SizeN
                });
            }
        }

        private void SelectNoneFilter()
        {
            foreach (Filter filter in Filters)
            {
                if (filter.IsChecked)
                    filter.IsChecked = false;
            }
        }

        private void SelectAllFilter()
        {
            foreach (Filter filter in Filters)
            {
                if (!filter.IsChecked)
                    filter.IsChecked = true;
            }
        }

        private void UnselectSelectedListElem()
        {
            if (!IsReplacingHappening)
            {
                SelectedListElem = null;
                PlacedItems[0].Visibility = "Hidden";
            }
        }

        private void UpdateAvailableListElems()
        {
            foreach (var elem in AllListElems)
                elem.IsAvailable = GamerMoney >= elem.Price;
        }
        #endregion Methods - Filtering, listing

        #region Methods - Map moving, zooming
        ///<Possible_development>
        ///ez a nagyítási módszer nem a legkifinomultabb, valami ZoomRatio szorzóval valószínűleg szebb lenne
        ///</Possible_development> 
        private void ZoomMap(int delta)
        {
            if (delta > 0 && RenderedTableWidth > 11)
            {
                --RenderedTableWidth;
                --ShiftUnit;
                PanelWidthHeightUnit = CanvasWidth / RenderedTableWidth;

                double yOffsetChange = CanvasHeight / 2 - CanvasMouseY;
                if (YOffset - yOffsetChange < 0)
                    YOffset = 0;
                else if (YOffset - yOffsetChange <= (TableHeight - RenderedTableHeight) * PanelWidthHeightUnit)
                    YOffset -= yOffsetChange;
                else
                    YOffset = (TableHeight - RenderedTableHeight) * PanelWidthHeightUnit;

                double xOffsetChange = CanvasWidth / 2 - CanvasMouseX;
                if (XOffset - xOffsetChange < 0)
                    XOffset = 0;
                else if (XOffset - xOffsetChange <= (TableWidth - RenderedTableWidth) * PanelWidthHeightUnit)
                    XOffset -= xOffsetChange;
                else
                    XOffset = (TableWidth - RenderedTableWidth) * PanelWidthHeightUnit;

                ShiftPlacedItems();
                if (SelectedListElem != null)
                    DrawHover();
            }
            else if (delta < 0 && (RenderedTableWidth < TableWidth /*|| RenderTableHeight < TableHeight*/))
            {
                ++RenderedTableWidth;
                ++ShiftUnit;
                PanelWidthHeightUnit = CanvasWidth / RenderedTableWidth;

                double yOffsetChange = CanvasHeight / 2 - CanvasMouseY;
                if (YOffset - yOffsetChange < 0)
                    YOffset = 0;
                else if (YOffset - yOffsetChange <= (TableHeight - RenderedTableHeight) * PanelWidthHeightUnit)
                    YOffset -= yOffsetChange;
                else
                    YOffset = (TableHeight - RenderedTableHeight) * PanelWidthHeightUnit;

                double xOffsetChange = CanvasWidth / 2 - CanvasMouseX;
                if (XOffset - xOffsetChange < 0)
                    XOffset = 0;
                else if (XOffset - xOffsetChange <= (TableWidth - RenderedTableWidth) * PanelWidthHeightUnit)
                    XOffset -= xOffsetChange;
                else
                    XOffset = (TableWidth - RenderedTableWidth) * PanelWidthHeightUnit;

                ShiftPlacedItems();
                if (SelectedListElem != null)
                    DrawHover();
            }
        }

        private void MoveMap(Key k)
        {
            if (k == Key.D) // Jobbra mozgatás
            {
                if (XOffset + ShiftUnit <= (TableWidth - RenderedTableWidth) * PanelWidthHeightUnit) // legfeljebb addig amennyi négyzet van
                {
                    XOffset += ShiftUnit; // nő az offset a shiftelés egységével
                    ShiftPlacedItems();
                    if (SelectedListElem != null)
                        DrawHover();
                }
                else if (XOffset < (TableWidth - RenderedTableWidth) * PanelWidthHeightUnit) // ha már egy egész offsetet hozzáadva túl sok lenne,
                {
                    XOffset = (TableWidth - RenderedTableWidth) * PanelWidthHeightUnit; // akkor a maximálisra állítjuk
                    ShiftPlacedItems();
                    if (SelectedListElem != null)
                        DrawHover();
                }
                else // ha már a maximálison állunk
                    return;
            }
            else if (k == Key.A) // Balra mozgatás
            {
                if (XOffset - ShiftUnit >= 0) //legfeljebb annyit hogy az offset ne menjen negatívba
                {
                    XOffset -= ShiftUnit;
                    ShiftPlacedItems();
                    if (SelectedListElem != null)
                        DrawHover();
                }
                else if (XOffset > 0) // ha már egy egész offsetet kivonva túl kisebb lenne mint a minimum (0), 
                {
                    XOffset = 0; // akkor 0-ra állítjuk
                    ShiftPlacedItems();
                    if (SelectedListElem != null)
                        DrawHover();
                }
                else // ha már a minimumon állunk
                    return;
            }
            else if (k == Key.S) // Lefele mozgatás
            {
                if (YOffset + ShiftUnit <= (TableHeight - RenderedTableHeight) * PanelWidthHeightUnit)// legfeljebb addig amennyi négyzet van
                {
                    YOffset += ShiftUnit; // nő az offset a shiftelés egységével
                    ShiftPlacedItems();
                    if (SelectedListElem != null)
                        DrawHover();
                }
                else if (YOffset < (TableHeight - RenderedTableHeight) * PanelWidthHeightUnit) // ha már egy egész offsetet hozzáadva túl sok lenne,
                {
                    YOffset = (TableHeight - RenderedTableHeight) * PanelWidthHeightUnit; //  akkor a maximálisra állítjuk
                    ShiftPlacedItems();
                    if (SelectedListElem != null)
                        DrawHover();
                }
                else // ha már a maximálisan állunk
                    return;
            }
            else if (k == Key.W) // Felfele mozgatás
            {
                if (YOffset - ShiftUnit >= 0) //legfeljebb annyit hogy az offset ne menjen negatívba
                {
                    YOffset -= ShiftUnit; // csökken az offset a shiftelés egységével
                    ShiftPlacedItems();
                    if (SelectedListElem != null)
                        DrawHover();
                }
                else if (YOffset > 0) // ha már egy egész offsetet kivonva túl kisebb lenne mint a minimum (0),
                {
                    YOffset = 0; // akkor 0-ra állítjuk
                    ShiftPlacedItems();
                    if (SelectedListElem != null)
                        DrawHover();
                }
                else // ha már a minimumon állunk
                    return;
            }
        }
        #endregion Methods - Map moving, zooming

        #region Methods - Item placing/replacing/rotation on high level
        private void DrawHover()
        {
            if (SelectedListElem != null)
            {
                if (_model.AreaEmpty(new Position { Y = TableMouseYWithOffset, X = TableMouseXWithOffset }))
                    PlacedItems[0].ImageSource = "Images/Good" + SelectedListElem.Type + ".png";
                else
                    PlacedItems[0].ImageSource = "Images/Bad" + SelectedListElem.Type + ".png";

                PlacedItems[0].PosYOnTable = TableMouseYWithOffset;
                PlacedItems[0].PosXOnTable = TableMouseXWithOffset;
                PlacedItems[0].Visibility = "Visible";

                PlaceItem(0);
            }
        }

        private void PlaceGame(int yCoordWithOffset, int xCoordWithOffset)
        {
            if (_model.AreaEmpty(new Position { Y = yCoordWithOffset, X = xCoordWithOffset }))
            {
                OpenGameSettingsDialogWindowEvent?.Invoke(this, EventArgs.Empty);
                if (IntCurrentGamePrice != -1 && IntCurrentGameMinCapacity != -1) // ez azért kell mert ha bezárta a beállító ablakot akkor ne helyezzünk le semmit
                {
                    _model.SetEntryPriceForCurrentElement(IntCurrentGamePrice);
                    _model.SetMinCapacityForCurrentElement(IntCurrentGameMinCapacity);

                    if (_model.PlaceEntity(new Position { Y = yCoordWithOffset, X = xCoordWithOffset }, PlacedItems.Count))
                    {
                        PlaceSelectedItem(yCoordWithOffset, xCoordWithOffset);
                        PlacedItems[^1].ImageSource = "Images/Building" + SelectedListElem.Type + ".png";

                        IntCurrentGameMinCapacity = -1;
                        IntCurrentGamePrice = -1;
                        DrawHover(); // azért kell mert zöld maradna felette
                        UpdateAvailableListElems();
                    }
                }
            }
        }

        private void PlaceService(int yCoordWithOffset, int xCoordWithOffset)
        {
            if (_model.AreaEmpty(new Position { Y = yCoordWithOffset, X = xCoordWithOffset }))
            {
                OpenServiceSettingsDialogWindowEvent?.Invoke(this, EventArgs.Empty);
                if (IntCurrentServicePrice != -1) // ez azért kell mert ha bezárta a beállító ablakot akkor ne helyezzünk le semmit
                {
                    _model.SetEntryPriceForCurrentElement(IntCurrentServicePrice);
                    if (_model.PlaceEntity(new Position { Y = yCoordWithOffset, X = xCoordWithOffset }, PlacedItems.Count))
                    {
                        PlaceSelectedItem(yCoordWithOffset, xCoordWithOffset);
                        PlacedItems[^1].ImageSource = "Images/Building" + SelectedListElem.Type + ".png";

                        IntCurrentServicePrice = -1;
                        DrawHover(); // azért kell mert zöld maradna felette
                        UpdateAvailableListElems();
                    }
                }
            }
        }

        private void PlaceStaff(int yCoordWithOffset, int xCoordWithOffset)
        {
            if (_model.AreaEmpty(new Position { Y = yCoordWithOffset, X = xCoordWithOffset }))
            {
                for (int i = 1; i < PlacedItems.Count; ++i) // kénytelen vagyok linsearch-csel megkeresni hogy melyik elemre kattintottak
                {
                    if (PlacedItems[i].PosYOnTable == yCoordWithOffset && PlacedItems[i].PosXOnTable == xCoordWithOffset)
                    {
                        if (_model.PlaceEntity(new Position { Y = yCoordWithOffset, X = xCoordWithOffset }, i))
                        {
                            PlacedItems[i].ImageSource = "Images/Happy" + SelectedListElem.Type + ".png";
                            if (IsReplacingHappening) // ha áthelyeztük
                            {
                                SelectedListElem = null; // akkor ne maradjon kiválasztva
                                IsReplacingHappening = false;
                                FilteringAndListingIsEnabled = true;
                                PlacedItems[0].Visibility = "Hidden";// és a hover ne jelenjen meg
                            }
                            else
                            {
                                DrawHover(); // azért kell mert zöld maradna felette
                                UpdateAvailableListElems();
                            }
                        }
                        break;
                    }
                }
            }
        }

        private void PlaceOther(int yCoordWithOffset, int xCoordWithOffset)
        {
            if (_model.AreaEmpty(new Position { Y = yCoordWithOffset, X = xCoordWithOffset }))
            {
                if (_model.PlaceEntity(new Position { Y = yCoordWithOffset, X = xCoordWithOffset }, PlacedItems.Count))
                {
                    PlaceSelectedItem(TableMouseYWithOffset, TableMouseXWithOffset);
                    switch (SelectedListElem.Type)
                    {
                        case TypeNameEnum.Generator:
                        case TypeNameEnum.Bin:
                        case TypeNameEnum.TreePlant:
                        case TypeNameEnum.FlowerPlant:
                        case TypeNameEnum.GrassPlant:
                            PlacedItems[^1].ImageSource = "Images/Operating" + SelectedListElem.Type + ".png";
                            break;
                        case TypeNameEnum.Road:
                            PlacedItems[^1].ImageSource = "Images/Clean" + SelectedListElem.Type + ".png";
                            break;
                        case TypeNameEnum.Guest:
                            break;
                        case TypeNameEnum.Cleaner:
                            break;
                        case TypeNameEnum.Mechanic:
                            break;
                        case TypeNameEnum.Toalett:
                            break;
                        case TypeNameEnum.HotDogRestaurant:
                            break;
                        case TypeNameEnum.IceCreamRestaurant:
                            break;
                        case TypeNameEnum.CottonCandyRestaurant:
                            break;
                        case TypeNameEnum.SushiRestaurant:
                            break;
                        case TypeNameEnum.RollerCoasterGame:
                            break;
                        case TypeNameEnum.GiantWheelGame:
                            break;
                        case TypeNameEnum.CarouselGame:
                            break;
                        case TypeNameEnum.MagicCastleGame:
                            break;
                        case TypeNameEnum.HorrorHouseGame:
                            break;
                        case TypeNameEnum.DodgemGame:
                            break;
                        case TypeNameEnum.MainGate:
                            break;
                        case TypeNameEnum.FenceBasic:
                            break;
                        case TypeNameEnum.FenceCorner:
                            break;
                        default:
                            throw new Exception("This shouldn't have happened! " +
                                "Such a type was selected for placing, which can't be placed by user");
                    }
                    DrawHover(); // azért kell mert zöld maradna felette
                    UpdateAvailableListElems();
                }
            }
        }

        private void ReplaceCleaner(int yCoordWithOffset, int xCoordWithOffset)
        {
            foreach (var elem in AllListElems)
            { // meg kell keresni a lista elemei közt mert így a legegyszerűbb az adatait megkapni
                if (elem.Type == TypeNameEnum.Cleaner)
                {
                    IsReplacingHappening = true; // és éppen áthelyezés fog történni
                    SelectedListElem = elem; // kiválasztjuk a Cleanert                                            
                    FilteringAndListingIsEnabled = false;
                    break;
                }
            }
            int i = 1;
            // linsearch-el megkeressük, hogy honnan akarunk áthelyezni
            for (; i < PlacedItems.Count; ++i)
            {
                if (PlacedItems[i].PosXOnTable == xCoordWithOffset && PlacedItems[i].PosYOnTable == yCoordWithOffset)
                {
                    PlacedItems[i].ImageSource = "Images/CleanRoad.png"; // ahonnan áthelyezzük sima út lesz
                    DrawHover();
                    break;
                }
            }
            if (i == PlacedItems.Count) // ha nem találtuk meg, akkor valami hiba van
                throw new Exception("Bug occured! Couldn't find the elem from where the cleaner would be replaced.");
        }

        private void RotateSelectedListElem()
        {
            if (SelectedListElem != null)
            {
                Tuple<int, int, int> newSizes = _model.RotateCurrentElement();
                PlacedItems[0].HeightOnTable = newSizes.Item2;
                PlacedItems[0].WidthOnTable = newSizes.Item1;
                PlacedItems[0].Rotation = 90 * newSizes.Item3;
                DrawHover();
            }
        }
        #endregion Methods - Item placing/replacing/rotation on high level

        #region Methods - Item placing/replacing/rotation on low level
        private void ShiftPlacedItems()
        {
            for (int i = 1; i < PlacedItems.Count; ++i)
                PlaceItem(i);
        }

        private void PlaceSelectedItem(int panelYWithOffset, int panelXWithOffset)
        {
            PlacedItems.Add(new PlacedItem());
            int i = PlacedItems.Count - 1;

            PlacedItems[i].IsHover = false;
            PlacedItems[i].PosYOnTable = panelYWithOffset;
            PlacedItems[i].PosXOnTable = panelXWithOffset;
            PlacedItems[i].HeightOnTable = PlacedItems[0].HeightOnTable;
            PlacedItems[i].WidthOnTable = PlacedItems[0].WidthOnTable;
            PlacedItems[i].Rotation = PlacedItems[0].Rotation;
            PlacedItems[i].Visibility = "Visible";

            PlaceItem(i);
        }

        private void PlaceItem(int indexOfPlacedItems)
        {
            PlacedItems[indexOfPlacedItems].PosY = PlacedItems[indexOfPlacedItems].PosYOnTable * (CanvasHeight / RenderedTableHeight) - YOffset;
            PlacedItems[indexOfPlacedItems].PosX = PlacedItems[indexOfPlacedItems].PosXOnTable * (CanvasWidth / RenderedTableWidth) - XOffset;
            PlacedItems[indexOfPlacedItems].Height = PlacedItems[indexOfPlacedItems].HeightOnTable * (CanvasHeight / RenderedTableHeight);
            PlacedItems[indexOfPlacedItems].Width = PlacedItems[indexOfPlacedItems].WidthOnTable * (CanvasWidth / RenderedTableWidth);
            PlacedItems[indexOfPlacedItems].Visibility = "Visible";
        }
        #endregion Methods - Item placing/replacing/rotation on low level

        #region Metods - Data modification
        private void ModifyGameDatas()
        {
            // lekérjük az eddigi adatot, hogy megjelenítsük az ablakban
            CurrentGameMinCapacity = _model.GetMinCapacityForCurrentElement().ToString();
            CurrentGamePrice = _model.GetEntryPriceForCurrentElement().ToString();
            OpenGameSettingsDialogWindowEvent?.Invoke(this, EventArgs.Empty);
            if (IntCurrentGamePrice != -1 && IntCurrentGameMinCapacity != -1) // ez azért kell mert ha bezárta a beállító ablakot akkor ne helyezzünk le semmit
            {
                _model.SetEntryPriceForCurrentElement(IntCurrentGamePrice);
                _model.SetMinCapacityForCurrentElement(IntCurrentGameMinCapacity);
                IntCurrentGameMinCapacity = -1;
                IntCurrentGamePrice = -1;
            }
        }

        private void ModifyServiceDatas()
        {
            // lekérjük az eddigi adatot, hogy megjelenítsük az ablakban
            CurrentServicePrice = _model.GetEntryPriceForCurrentElement().ToString();
            OpenServiceSettingsDialogWindowEvent?.Invoke(this, EventArgs.Empty);
            if (IntCurrentServicePrice != -1)
            {
                _model.SetEntryPriceForCurrentElement(IntCurrentServicePrice);
                IntCurrentServicePrice = -1;
            }
        }

        private void ModifyMainGateDatas()
        {
            // lekérjük az eddigi adatot, hogy megjelenítsük az ablakban
            ParkEntryPrice = _model.GetEntryPriceForCurrentElement().ToString();
            OpenParkSettingsDialogWindowEvent?.Invoke(this, EventArgs.Empty);
            if (IntParkEntryPrice != -1)
                _model.SetEntryPriceForCurrentElement(IntParkEntryPrice);
        }
        #endregion Metods - Data modification

        #region Methods - Validation of incoming datas
        private void CloseGameSettingsDialogWindow()
        {
            if (int.TryParse(CurrentGamePrice, out int converted) && converted >= 0 &&
                int.TryParse(CurrentGameMinCapacity, out int converted2) && converted2 >= 0 && converted2 <= 100)
            {
                IntCurrentGamePrice = converted;
                IntCurrentGameMinCapacity = converted2;
                CloseGameSettingsDialogWindowEvent?.Invoke(this, EventArgs.Empty);
            }
            else
            {
                ErrorEvent?.Invoke(this, "The ticket price as well as the minimum capacity must be a nonnegative integer number. " +
                    "The minimum capacity must be less or equal than 100.");
            }
        }

        private void CloseServicePriceDialogWindow()
        {
            if (int.TryParse(CurrentServicePrice, out int converted) && converted >= 0)
            {
                IntCurrentServicePrice = converted;
                CloseServiceSettingsDialogWindowEvent?.Invoke(this, EventArgs.Empty);
            }
            else
            {
                ErrorEvent?.Invoke(this, "The price must be a nonnegative integer number.");
            }
        }

        private void CloseParkSettingsDialogWindow()
        {
            if (int.TryParse(ParkEntryPrice, out int converted) && converted >= 0)
            {
                IntParkEntryPrice = converted;
                CloseParkSettingsDialogWindowEvent?.Invoke(this, EventArgs.Empty);
            }
            else
            {
                ErrorEvent?.Invoke(this, "The entry ticket price must be a nonnegative integer number.");
            }
        }
        #endregion Methods - Validation of incoming datas

        #region Methods - Others
        private void NewGame()
        {
            _model.NewGame();
            TableWidth = _model.GetSize().Item1; //sizeN - X tengely
            TableHeight = _model.GetSize().Item2; //sizeM - Y tengely
            PlacedItems = new ObservableCollection<PlacedItem>
            {
                new PlacedItem { IsHover = true, Visibility = "Hidden" } // az első kitüntett lesz, ez lesz a hover
            };
            GameState = "Closed";
            GamerMoney = _model.GetMoney();
            StatusBar = "Money: " + String.Format("{0:n0}", GamerMoney) + " Ft | Park is: " + GameState;
            IsReplacingHappening = false;
            IntCurrentGameMinCapacity = -1;
            IntCurrentGamePrice = -1;
            IntCurrentServicePrice = -1;
            IntParkEntryPrice = -1;
            ItemExtraDetailsToolTipVisibility = "Hidden";
            ItemExtraDetailsToolTipText = "";
            FilteringAndListingIsEnabled = true;
            SelectedListElem = null;
            UpdateAvailableListElems();

            RenderedTableWidth = 25; // a setterében beállítja hozzá kiszámolva a RenderTableHeight-ot
            ShiftUnit = 30;

            // bal alulra középre tegyük a kezdőpozíciót, itt lesz a főkapu
            // Továbbfejlesztési lehetőség, hogy ez ne legyen beégetve, hanem a modelltől jöjjön
            XOffset = ((TableWidth - RenderedTableWidth) * PanelWidthHeightUnit) / 2;
            YOffset = (TableHeight - RenderedTableHeight) * PanelWidthHeightUnit;
            
            BuildTable();
        }

        private void BuildTable()
        {
            var list = _model.GetInfrastructures();
            foreach (var elem in list)
            {
                PlacedItems.Add(new PlacedItem
                {
                    PosYOnTable = elem.position.Y,
                    PosXOnTable = elem.position.X,
                    HeightOnTable = elem.Identity.SizeM,
                    WidthOnTable = elem.Identity.SizeN,
                    Rotation = 90 * elem.Identity.ConstructionInformation.RotationTimes
                });
                switch (elem.Identity.Type)
                {
                    case TypeNameEnum.Guest:
                        break;
                    case TypeNameEnum.Cleaner:
                        break;
                    case TypeNameEnum.Mechanic:
                        break;
                    case TypeNameEnum.Bin:
                        break;
                    case TypeNameEnum.TreePlant:
                        break;
                    case TypeNameEnum.FlowerPlant:
                        break;
                    case TypeNameEnum.GrassPlant:
                        break;
                    case TypeNameEnum.Road:
                        PlacedItems[^1].ImageSource = "Images/CleanRoad.png";
                        break;
                    case TypeNameEnum.Toalett:
                        break;
                    case TypeNameEnum.HotDogRestaurant:
                        break;
                    case TypeNameEnum.IceCreamRestaurant:
                        break;
                    case TypeNameEnum.CottonCandyRestaurant:
                        break;
                    case TypeNameEnum.SushiRestaurant:
                        break;
                    case TypeNameEnum.RollerCoasterGame:
                        break;
                    case TypeNameEnum.GiantWheelGame:
                        break;
                    case TypeNameEnum.CarouselGame:
                        break;
                    case TypeNameEnum.MagicCastleGame:
                        break;
                    case TypeNameEnum.HorrorHouseGame:
                        break;
                    case TypeNameEnum.DodgemGame:
                        break;
                    case TypeNameEnum.Generator:
                        break;
                    case TypeNameEnum.MainGate:
                        PlacedItems[^1].ImageSource = "Images/MainGate.png";
                        break;
                    case TypeNameEnum.FenceBasic:
                        PlacedItems[^1].ImageSource = "Images/FenceBasic.png";
                        break;
                    case TypeNameEnum.FenceCorner:
                        PlacedItems[^1].ImageSource = "Images/FenceCorner.png";
                        break;
                    default:
                        break;
                }
                PlaceItem(PlacedItems.Count - 1);
            }
        }      

        private void UpdateDetails()
        {
            try
            {
                var datas = _model.GetEntitiyInfoForPosition(new Position(TableMouseXWithOffset, TableMouseYWithOffset));
                if (datas == null)
                {
                    ItemExtraDetailsToolTipText = "";
                    ItemExtraDetailsToolTipVisibility = "Hidden";
                    return;
                }

                switch (datas.Category)
                {
                    case CategoryEnum.Trash:
                    case CategoryEnum.Plant:
                    case CategoryEnum.Decoration:
                    case CategoryEnum.Bin:
                    case CategoryEnum.Road:
                        ItemExtraDetailsToolTipText = "";
                        ItemExtraDetailsToolTipVisibility = "Hidden";//ezekhez semmit nem akarunk kiírni  
                        break;

                    // Továbbfejlesztési lehetőség: ezekhez is ki lehetne írni valami kis információt akár
                    case CategoryEnum.Person:
                    case CategoryEnum.Guest:
                    case CategoryEnum.Staff:
                    case CategoryEnum.Cleaner:
                    case CategoryEnum.Mechanic:
                    case CategoryEnum.Generator:
                    case CategoryEnum.MainGate:
                        ItemExtraDetailsToolTipText = "";
                        ItemExtraDetailsToolTipVisibility = "Hidden";
                        break;


                    case CategoryEnum.Toalett:
                    case CategoryEnum.Restaurant:
                        ItemExtraDetailsToolTipVisibility = "Visible";

                        ItemExtraDetailsToolTipText = "Capacity: " + datas.MaximumCapacity + " \n";
                        ItemExtraDetailsToolTipText += "Ticket: " + datas.TicketPrice + " Ft\n";
                        ItemExtraDetailsToolTipText += "Always cost: " + datas.AlwaysCost + " Ft\n";
                        ItemExtraDetailsToolTipText += "Single cost: " + datas.SingleCost + " Ft\n";
                        ItemExtraDetailsToolTipText += "Queue: " + datas.PeopleWaiting + " \n";
                        ItemExtraDetailsToolTipText += "Using: " + datas.PeopleUsing; 


                        break;
                    case CategoryEnum.Game:
                        ItemExtraDetailsToolTipVisibility = "Visible";
                        ItemExtraDetailsToolTipText = "Capacity: " + datas.MaximumCapacity + " \n";
                        ItemExtraDetailsToolTipText += "Minumum: " + datas.MinimumCapacity + "\n";
                        ItemExtraDetailsToolTipText += "Ticket: " + datas.TicketPrice + " Ft\n";
                        ItemExtraDetailsToolTipText += "Always cost: " + datas.AlwaysCost + " Ft\n";
                        ItemExtraDetailsToolTipText += "Single cost: " + datas.SingleCost + " Ft\n";
                        ItemExtraDetailsToolTipText += "Queue: " + datas.PeopleWaiting + " \n";
                        ItemExtraDetailsToolTipText += "On game: " + datas.PeopleUsing ; 
                        break;

                    case CategoryEnum.Building:
                    case CategoryEnum.Infrastructure:
                    case CategoryEnum.Other:
                    default:
                        throw new Exception("Bug occured! Given category which is not handled!");
                }
            }
            catch (Exception) { ItemExtraDetailsToolTipVisibility = "Hidden"; ItemExtraDetailsToolTipText = ""; }
        }

        private void OpenPark()
        {
            if (IntParkEntryPrice == -1) // Ha még nem állította be a főkapura kattintással, akkor feldobjuk a beállító ablakot
                OpenParkSettingsDialogWindowEvent?.Invoke(this, EventArgs.Empty);
            _model.StartGame(IntParkEntryPrice);
            GameState = "Open";
        }
        #endregion Methods - Others
    }
}
